//мк ATtiny13a
//F_CPU = 9600000

#include <avr/io.h>
#include <avr/interrupt.h>


//Обьявление типа unsigned char новым названием uchar (для дальнейшего использования в коде)
typedef unsigned char uchar;

#define RX PB4			//RX - для приема
#define SPEED 9600		//Скорость передачи
#define DISP 0		    //Подстройка скорости; -30/+30



/*
*	Начать передачу с отсчета до половины периода.
*	TCCR0B |= 0b00000010; - включить таймер с предделителем на 8
*	PCMSK &= ~(1 << RX);  - запретить прерывания PCINT для RX на время передачи
*/
#define RECEIVE_START \
	OCR0A = ((F_CPU/8) / SPEED - 1 + DISP)/2; \
	TCCR0B &= 0b11111000; \
	TCCR0B |= 0b00000010; \
	PCMSK &= ~(1 << RX);

/*
*	Завершить передачу, отключить таймер, и включить прерывания PCINT на RX
*/
#define RECEIVE_END			  \
		TCCR0B &= 0b11111000; \
		TCNT0 = 0;			  \
		PCMSK |= (1 << RX);


static void bit_received(void);
static void byte_received(uchar data);


ISR(PCINT0_vect, ISR_BLOCK) {
	if(bit_is_clear(PINB, RX))		//описать в тексте
		
		RECEIVE_START;
}


//Обработчик прерывания таймера по совпадению
//ISR_BLOCK - блокирует глобальный флаг разрешения прерываний во время выполнения обработчика
ISR(TIM0_COMPA_vect, ISR_BLOCK) {
	bit_received();
}


int main(void) {
    DDRB =  0b00001111;			//PB1-PB4 - выходы
    PORTB = 0b11110000;
	GIMSK = (1 << PCIE);
	PCMSK = (1 << RX);			//PCINT на RX
	
	//OSCCAL = 59;              //?????????

	TCCR0A = 0b00000010;		//Режим - сброс по совпадению (CTC)
	TCCR0B = 0b00000000;
	TIMSK0 = (1 << OCIE0A); 	//Разрешение прерывания по совпадению с OCR0A 
    
	sei();
	while(1) {}
}


void bit_received(void) {
	static uchar bit_count = 0;			//счетчик полученных битов
	static uchar buf = 0;				//буфер для записи полученной информации

	if(bit_count == 0) {				//Стартовый бит
		if(bit_is_clear(PINB, RX)) {	//Проверка, точно ли это был статовый бит
			buf = 0;
			bit_count++;
			OCR0A = (F_CPU/8) / SPEED - 1 + DISP;
		}
		else {							//Если все же помеха, то закончить прием
			RECEIVE_END;
			bit_count = 0;
		}
	}
	else if(bit_count == 9) {			//Стоповый бит, конец передачи байта
		RECEIVE_END;
		bit_count = 0;
		byte_received(buf);				//Обработка полученного байта
	}
	else {								//Один из информационных битов
		buf >>= 1;
		if(bit_is_set(PINB, RX))
			buf |= 0b10000000;
		bit_count++;
	}
}



void byte_received(uchar data) {
	sei();
    PORTB &= 0b111100000;
    PORTB |= 0b00001111 & (data);
}
